/* Write a short program in which the process with rank 0 sends the same
 * message to all other processes in MPI_COMM_WORLD and then receives
 * a message of the same length from all other processes. How does the time
 * taken varies with the size of the messages and with the number of processes?
 * Assume the format of messages same as in part (1).
 *
 *
 */


/* Rank 0 send the message to all of the other ranks
 *
 * Input: Variable size of the message
 * Output:  the communication time, the variable size, the transfer rate
 *
 */

#include <stdio.h> 
#include <stdlib.h>

/* using MPI routines, definitions, etc. */
#include "mpi.h"

#define ping 101  /* Tag for SEND */
#define pong 101  /* Tag for RECEIVE */

main(int argc, char** argv) {

    int         each_process_rank;   /* Each process rank */
    int         process_num;         /* The number of processes */

    int         process_source;    /* Process sending the variable size  */
    int         process_dest = 0;  /* All messages go to Process 0 */
    int         length;           /* The length of the message */
    float       buffer[100000];   /* The max size of the message */
    double      start, finish, time;  /* For checking time variable */
    int i;   /* for loop variable */

    MPI_Status  status;

    /* Let the system do what it needs to start up MPI */
    MPI_Init(&argc, &argv);

    /* Get each process rank */
    MPI_Comm_rank(MPI_COMM_WORLD, &each_process_rank);

    /* Find out how many processes are being used */
    MPI_Comm_size(MPI_COMM_WORLD, &process_num);

    /* The part of each process */
    if (each_process_rank == 0) {

	printf("Length\tTotal Time\tTransfer Rate\n");

/* the Variable size of the message processing rank 0 */

    	for (length = 1; length <=  100000; length += 1000){
        	start = MPI_Wtime();  /* For checking MPI time */
 
 	/* The iteration number for getting the average time */
	       	for (i = 1; i <= 100; i++){
 
	/* Send the message to each processes */
        		for (process_source = 1; process_source < process_num; 
					  process_source++) {
 
            		MPI_Send(buffer, length, MPI_FLOAT, process_source, ping,
                                 MPI_COMM_WORLD);
				}

	/* Receive the message from each processes */
			for (process_source = 1; process_source < process_num;
					  process_source++) {

            		MPI_Recv(buffer, length, MPI_FLOAT, process_source, pong,
                    			 MPI_COMM_WORLD, &status);
        		}
		 }

        	finish = MPI_Wtime();  /* For checking the MPI time */
 
        	time = finish - start; /* The result of the time */
 
        	printf("%d\t%f\t%f\n", length, time/200.,
               (float)(2 * 8 * 100 * length)/time);
 
    	}

    } else {  

	/* The variable size of the message processing */

    		for (length = 1; length <= 100000; length += 1000) {

	/* The iteration of getting the average time */ 
            	for (i = 1; i <= 100; i++) {
 
	/* Each processes except rank 0 sends and receives the messages */
                	MPI_Recv(buffer, length, MPI_FLOAT, process_dest, ping,
                                             MPI_COMM_WORLD, &status);
 
                	MPI_Send(buffer, length, MPI_FLOAT, process_dest, pong,
                                            MPI_COMM_WORLD);
 
            	}
    		}
    }

    MPI_Finalize();
} /*  main  */

